# PDDL modelling 

You must read fully and carefully the assignment specification and instructions detailed in this file. You are NOT to modify this file in any way.

The **aim of this project** is to improve your understanding of PDDL modelling.

## Task

**Your task** is to read the description of the domain below and encode the three planning problem instances specified filling the template files provided in this repository. Finally, you will be asked to extend your solution to deal with an updated spec.

## The Domain - Real Player Game Scenario

 *Help the hero to get out of dungeon!*
 
 A hero woke up in a dungeon full of monsters and traps (perhaps the party last night went wrong...) and needs your help to get out.  Here are basic facts for the dungeon domain:
- The dungeon contains rooms that are **connected** by corridors (dungeon can thus be represented by undirected graph)
- each room can be **empty**, or can have a **monster** in it, or can have a **trap** in it, or can have a **sword** in it
- one of the empty rooms is the **goal**: it has an exit, so the hero can escape

The hero is lucky since she/he has full knowledge about the dungeon. But not that lucky – just after the hero leaves each room s/he visits, the room is **destroyed** and cannot be visited again.

The hero can perform the following actions – but only if s/he is alive!
- The hero can **move** to an adjacent room (connected by a corridor) that has not been destroyed (i.e., the hero has not already visited the room)
- **Pickup** the sword if present in the room the hero is currently in and the hero is empty handed
- **Destroy** the sword that the hero currently holds. However, this can have unpleasant effects if done in a room with a trap or a monster: it triggers the trap or invites the monster to eat the hero.
- **Disarm a trap** – if there is a trap in the room the hero is in and the hero is empty-handed (does not hold a sword), then the hero can disarm it

However, there are some (dangerous) constraints the hero has to consider:
- If the hero enters a room with a monster in it, s/he has to carry a sword (so the monster is afraid of him/her), otherwise the monster kills him/her. Notice that the hero is pacifist, so s/he cannot kill the monster.
- If the hero destroys the sword in a room with a monster in it, the monster kills him/her as well.
- The only action the hero can safely perform in a room with a trap in it is the **disarm a trap** action. Any other action (even moving away) triggers the trap which kills the hero.

**Important:** We recommend reading the description carefully several times to make sure you fully understand it before starting coding in PDDL.

### Problem 1

The problems are specified such that cells stand for rooms and edges between them represent corridors. **I** is the hero's initial position, **G** is hero's desired goal position, **$S_i$** indicates a sword, **M** is a monster, and **T** stands for trap. The numbers identify a unique ID for each cell. Good luck!

![problem1](problem1.png)


### Problem 2

![problem1](problem2.png)


### Problem 3

![problem1](problem3.png)

# Planimated Example Solution
![Planimation](planimation.gif)

You can generate the gif above to visualise your solutions, but you will need to use the planimation plugin in the editor.planning.domains, the animation file, and you will need to add a *cell* predicate to the domain and problem files, so that the animation engine can distribute the cells in a grid accordingly. This is not mandatory, it's just optional in case you want to visualise your solutions. Happy to discuss this further in ED.

# Problem 4

Once you modeled correctly the three problems, please consider how would you change your PDDL domain and problem to make it **turn based**, where at each turn the planner chooses an action for each hero present in the dungeon. For this problem, you can assume a fixed number of heroes greater than 1, and only one hero can be in the same location. You will likely need to add or change existing predicates and actions. Include a problem instance with more than one hero where all of them can reach their respective goals. Once a hero reaches a target goal location, you can assume there's no need for the planner to assign an action for that hero.

## Marking Criteria - Self Evaluation

The criteria for marking will be:

- 3 points for encoding the domain correctly. Each correct action counts 0.5 points. If a precondition or effect is missing from an action, the action will earn 0 points.
- 1 point for encoding the 3 problems correctly
- 1 point for generating correct solutions as per the specification of the problem
- 5 points for encoding problem 4 correctly as per specification 
   - 1 point for handling more than one hero, 
   - 2 points for making it turn based,
   - 2 points for creating a problem with a valid solution for all heroes.

Change the domain and problem files.

### Self-Evaluation
For problems 1-3 (together) and problem 4, write a brief self evaluation that includes how many marks you believe that you have earnt in ths assignment. Your self evaluation must include:

* Self evaluated marks for each question, with a breakdown of:

    * 3 marks for domain file (problem 1-3)
    * 1 marks for problem files (Problems 1-3)
    * 1 marks for plan correctness
    - Problem 4:
        * 1 mark for handling 2 heros
        * 2 marks for making it turn based
        * 2 marks for creating a solvable problem

* A brief overview of why you have earnt this. As part of your overview, consider the following:
    
    * whether you can successfully generate correct plans for the goals;
    * explain how you checked the correctness of your plans;
    * whether you believe that your model is robust -- that is, whether it can generate correct plans for goals other than those specified above and why you believe it is robust;
    * whether your model is clear, concise, and at the 'right' level of abstraction -- that is, the model contains only those aspects that are needed to solve the problem;
    * what you learnt about modelling and abstraction; and
    * list any assumptions that you have made.
       
For each part (problem 1-3 and problem 4), keep this to 2-3 paragraphs.

Note that completing the tasks without submitting a self evaluation will not attract ANY marks. You must tell us the evidence that you self-evaluate yourself.

**Important:** Make sure to tag your commit with your updated pddl files inside that folder.

## Ambiguous details?

You may have questions about the above description, such as: "what should happen if this happens, but then that -- what should the behaviour be?".

The answer: just choose a sensible behaviour and model it. We don't care if the model does exactly what our model does -- we care that you are learning about modelling and abstraction. You should be able to make a sensible assumption about the behaviour and model it. You will still learn the same amount!

## Debugging tips and other resources

If the planner is failing to return a plan, but you think it should, debugging is more difficult than in imperative programming languages, because the model is declarative and there are no standard PDDL debuggers.

There are two techniques to help debug the model:

1. **Simplify the goal**: If your goal contains multiple predicates, try removing them systematically to see which goal (or combinations of goals) are preventing the solution. This helps to identify which actions you may have modelled incorrectly.

2. **Use preconditions as goals**: If tip 1 does not work, write out the plan that you expect it to find. Then, create a new problem instance, take the first action of your expected plan, and set its precondition as the goal. If it fails, you have identified where your model is not doing what you expect. If it does not fail, move to next action in the plan, setting its precondition as the goal, and iterate.
